@@ -26,14 +26,14 @@ class Enclosure : public QObject
     Q_PROPERTY(qint64 sizeOnDisk READ sizeOnDisk NOTIFY sizeOnDiskChanged)
     Q_PROPERTY(QString title MEMBER m_title CONSTANT)
     Q_PROPERTY(QString type MEMBER m_type CONSTANT)
-    Q_PROPERTY(QString url MEMBER m_url CONSTANT)
+    Q_PROPERTY(QString url READ url CONSTANT)
     Q_PROPERTY(Status status READ status WRITE setStatus NOTIFY statusChanged)
     Q_PROPERTY(double downloadProgress MEMBER m_downloadProgress NOTIFY downloadProgressChanged)
     Q_PROPERTY(QString formattedDownloadSize READ formattedDownloadSize NOTIFY downloadProgressChanged)
     Q_PROPERTY(QString path READ path CONSTANT)
     Q_PROPERTY(qint64 playPosition READ playPosition WRITE setPlayPosition NOTIFY playPositionChanged)
     Q_PROPERTY(QString formattedLeftDuration READ formattedLeftDuration NOTIFY playPositionChanged)
-    Q_PROPERTY(QString formattedPlayPosition READ formattedPlayPosition NOTIFY playPositionChanged);
+    Q_PROPERTY(QString formattedPlayPosition READ formattedPlayPosition NOTIFY playPositionChanged)
     Q_PROPERTY(qint64 duration READ duration WRITE setDuration NOTIFY durationChanged)
     Q_PROPERTY(QString formattedDuration READ formattedDuration NOTIFY durationChanged)
 
@@ -56,6 +56,7 @@ public:
     Q_INVOKABLE void deleteFile();
 
     QString path() const;
+    QString url() const;
     Status status() const;
     qint64 playPosition() const;
     qint64 duration() const;
@@ -89,7 +90,7 @@ private:
     Entry *m_entry;
     qint64 m_duration;
     qint64 m_size = 0;
-    qint64 m_sizeOnDisk;
+    qint64 m_sizeOnDisk = 0;
     QString m_title;
     QString m_type;
     QString m_url;
